@@ -5,14 +5,14 @@
 ##' @inheritParams geom_pokemon
 ##' @return ggplot2 layer
 ##' @export
-##' @author guangchuang yu
+##' @author Guangchuang Yu
 geom_icon <- function(mapping = NULL, data = NULL, inherit.aes = TRUE,
                       na.rm = FALSE, by = "width", ...) {
     geom_image(mapping, data, inherit.aes=inherit.aes, na.rm=na.rm, ..., .fun = icon)
 }
 
 icon <- function(id) {
-    ## paste0('https://raw.githubusercontent.com/ionic-team/ionicons/master/src/svg/', id, '.svg')
+    ## paste0('https://raw.githubusercontent.com/ionic-team/ionicons/main/src/svg/', id, '.svg')
     url <- paste0('https://ionicons.com/ionicons/svg/', id, '.svg')
 
     check_url(url)
@@ -24,7 +24,9 @@ icon <- function(id) {
 ##' @title list.icon
 ##' @return icon vector
 ##' @export
-##' @author guangchuang yu
+##' @author Guangchuang Yu
 list.icon <- function() {
-    list.png("https://github.com/ionic-team/ionicons/tree/master/src/svg")
+    # list.png("https://github.com/ionic-team/ionicons/tree/main/src/svg")
+    files <- get_github_files("ionic-team", "ionicons", "src/svg", "main")
+    sub(".svg", "", files)
 }
